@@ -4,26 +4,32 @@
 #'
 #' @param data Input dataset. Must contain a dependent and a set of independent variables.
 #' @param yvar Name of the dependent variable, enclosed in quotation marks. If NULL, the first variable will be used.
-#' @param Xvar Vector of names of the independent variables, each enclosed in quotation marks. If NULL, all variables except the first will be used.
-#' @param difflags Fixed lagged differences for the short term part of the ARDL equation.
-#' @param maxlag Max number of lags for the auto_ardl procedure.
-#' @param p.ardl Threshold p-value for the short-term ARDL coefficients significance in the bootstrap procedure.
-#' @param p.vecm Threshold p-value for the short-term VECM coefficients significance in the bootstrap procedure.
-#' @param B Number of bootstrap replications.
+#' @param xvar Vector of names of the independent variables, each enclosed in quotation marks. If NULL, all variables except the first will be used.
+#' @param fix.ardl Fixed lagged differences for the short term part of the ARDL equation.
+#' @param info.ardl Selection criterion for the auto_ardl function. Options are "AIC", "AICc", BIC, "R2", "adjR2", if fix.ardl is null. Defaults to AIC.
+#' @param fix.vecm Fixed lagged differences for the short term part of the VECM equation.
+#' @param info.vecm Selection criterion for the VARselect function. Options are "AIC", "HQ", "SC", "FPE", if fix.vecm is null. Defaults to AIC.
+#' @param maxlag Max number of lags for the auto_ardl  and VARselect procedures, if fix.ardl or fix.vecm are null, respectively.
+#' @param a.ardl Threshold significance for the short-term ARDL coefficients significance.
+#' @param a.vecm Threshold significance for the short-term VECM coefficients significance.
+#' @param nboot Number of bootstrap replications.
 #' @param case Model case, pertaining to the treatment of intercept and trend. Must be integer from 1 to 5. Defaults to 3.
-#' @param crit.H0 Probability/ies by which the critical quantiles of the bootstrap distribution(s) must be calculated.
+#' @param a.boot.H0 Probability/ies by which the critical quantiles of the bootstrap distribution(s) must be calculated.
 #' @param print Show the progress bar.
-
 #' @return List of several elements including \itemize{
-#'\item \code{ARDL}: the conditional and unconditional ARDL models applied on the data
+#'\item \code{data}: the data used to perform estimation and testing
+#'\item \code{ARDL}: the estimated ARDL conditional model
+#'\item \code{VECM}: the estimated VECM unconditional model
+#'\item \code{jo.testX}: Johansen cointegration test on the independent variables
 #'\item \code{pssbounds}: the PSS bound test output
 #'\item \code{smgbounds}: the SMG bound test critical values
-#'\item \code{fov.st}: the test statistics on the conditional and unconditional Fov tests
-#'\item \code{t.st}: the test statistics on the conditional and unconditional t tests
-#'\item \code{find.st}: the test statistics on the conditional and unconditional Find tests
-#'\item \code{quantFOV}: the bootstrap conditional and unconditional F Overall test critical value(s)
-#'\item \code{quantt}: the bootstrap conditional and unconditional t test critical value(s)
-#'\item \code{quantFIND}: the bootstrap conditional and unconditional F Independent test critical value(s)}
+#'\item \code{fov.stat}: the test statistics on the conditional Fov tests
+#'\item \code{t.stat}: the test statistics on the conditional t test
+#'\item \code{find.stat}: the test statistics on the conditional Find tests
+#'\item \code{quantfov}: the bootstrap conditional F Overall test critical value(s)
+#'\item \code{quantt}: the bootstrap conditional t-test critical value(s)
+#'\item \code{quantfind}: the bootstrap conditional F Independent test critical value(s)
+#'\item \code{fakecoint}: indication of the situation in which \eqn{a_{y.x}\neq 0} but \eqn{a_{y.x}^{UC}=0}, signaling absence of cointegration.}
 #'
 #' @examples
 #'\dontrun{
@@ -31,21 +37,26 @@
 #' LNDATA = as.data.frame(log(ger_macro[,-1]))
 #' colnames(LNDATA) = c("LNINVEST","LNINCOME","LNCONS")
 #'
-#' boot_res = boot_ardl(LNDATA, yvar = "LNINCOME", Xvar = c("LNCONS","LNINVEST"), maxlag = 5, B = 2000)
+#' boot_res = boot_ardl(data=LNDATA,
+#'                      yvar = "LNINCOME", xvar = c("LNCONS","LNINVEST"),
+#'                      maxlag = 5, nboot = 2000)
 #' summary(boot_res)
 #'}
 #' @export
 boot_ardl =
   function(data,
            yvar = NULL,
-           Xvar = NULL,
-           difflags = NULL,
+           xvar = NULL,
+           fix.ardl = NULL,
+           info.ardl = "AIC",
+           fix.vecm = NULL,
+           info.vecm = "AIC",
            maxlag = 5,
-           p.ardl = 0.05,
-           p.vecm = 0.05,
-           B = 2000,
+           a.ardl = 0.05,
+           a.vecm = 0.05,
+           nboot = 2000,
            case = 3,
-           crit.H0 = c(0.05, 0.025, 0.01),
+           a.boot.H0 = c(0.05, 0.025, 0.01),
            print = T) {
 
     if(!any(inherits(data,"data.frame"))){
@@ -58,55 +69,66 @@ boot_ardl =
     if (is.null(yvar)) {
       yvar = dfnames[1]
     }
-    if (is.null(Xvar)) {
-      Xvar = dfnames[-1]
+    if (is.null(xvar)) {
+      xvar = dfnames[-1]
     }
 
     if (!(yvar %in% dfnames)) {
-      stop("y variable not in dataset")
+      stop("y variable not in dataset.")
     }
 
-    if (!all(Xvar %in% dfnames)) {
+    if (!all(xvar %in% dfnames)) {
       stop("at least one of the X variables not in dataset.")
     }
 
-    if (yvar %in% Xvar) {
+    if (yvar %in% xvar) {
       stop("y variable and X variables must be separated.")
     }
 
-    if (p.ardl < 0 | p.ardl > 1) {
+    if (a.ardl < 0 | a.ardl > 1) {
       warning("Invalid ARDL threshold p.value. Defaulting to 0.05.")
-      p.ardl = 0.05
+      a.ardl = 0.05
     }
 
-    if (p.vecm < 0 | p.vecm > 1) {
+    if (a.vecm < 0 | a.vecm > 1) {
       warning("Invalid VECM threshold p.value. Defaulting to 0.05.")
-      p.vecm = 0.05
+      a.vecm = 0.05
     }
 
-    if (length(p.ardl) > 1) {
-      warning("Expecting a scalar value. Defaulting to 0.05.")
-      p.ardl = 0.05
+    if (length(a.ardl) > 1) {
+      warning("Expecting a scalar a.ardl value. Defaulting to 0.05.")
+      a.ardl = 0.05
     }
 
-    if (length(p.vecm) > 1) {
-      warning("Expecting a scalar value. Defaulting to 0.05.")
-      p.vecm = 0.05
+    if(length(a.vecm) > 1) {
+      warning("Expecting a scalar a.vecm value. Defaulting to 0.05.")
+      a.vecm = 0.05
     }
 
-    if (any(difflags %% 1 != 0)) {
-      warning("Expecting integer values. Rounding to the nearest integer.")
-      difflags = round(difflags)
+    if(!is.null(fix.ardl)) {
+      if(any(fix.ardl %% 1 != 0)){
+      warning("Expecting integer fix.ardl values. Rounding to the nearest integer.")
+      fix.ardl = round(fix.ardl)
+      }
     }
 
+    if (!is.null(fix.vecm)) {
+      if(fix.vecm %% 1 != 0){
+      warning("Expecting integer fix.vecm value. Rounding to the nearest integer.")
+      fix.vecm = round(fix.vecm)
+      }
+    }
+
+    if (!is.null(maxlag)) {
     if (any(maxlag %% 1 != 0)) {
-      warning("Expecting integer value. Rounding to the nearest integer.")
+      warning("Expecting integer maxlag value. Rounding to the nearest integer.")
       maxlag = round(maxlag)
     }
+    }
 
-    if (B %% 1 != 0) {
-      warning("Expecting integer value. Rounding to the nearest integer.")
-      B = round(B)
+    if (nboot %% 1 != 0) {
+      warning("Expecting integer nboot value. Rounding to the nearest integer.")
+      nboot = round(nboot)
     }
 
     if (case < 1 | case > 5 | case %% 1 != 0) {
@@ -114,16 +136,20 @@ boot_ardl =
       case = 3
     }
 
-    if (any(crit.H0 < 0) | any(crit.H0 > 1)) {
+    if (any(a.boot.H0 < 0) | any(a.boot.H0 > 1)) {
       warning("Invalid probability for critical values. Defaulting to 1%, 2.5% and 5%")
-      crit.H0 = c(0.01, 0.025, 0.05)
+      a.boot.H0 = c(0.01, 0.025, 0.05)
     }
 
-    if(B<200){
-      warning("suppressing print: B should be at least 200")
+    if(nboot<200){
+      warning("Suppressing print: nboot should be at least 200")
     }
 
-    numdata = data %>% dplyr::select(yvar, Xvar)
+    #SETTING FLAGS FOR SPURIOUS COINTEGRATION
+    flagspur = rep(0,length(a.boot.H0))
+    computeuc = 0
+    
+    numdata = data %>% dplyr::select(all_of(yvar), all_of(xvar))
     d = ncol(numdata)
 
     #LAGGED AND FIRST DIFFERENCE
@@ -131,57 +157,84 @@ boot_ardl =
     dlag0 = numdata - lagdata
     colnames(dlag0) <- paste("D_", colnames(numdata), sep = "")
 
-    if (is.null(difflags)) {
-      #INITIAL ARDL MODEL DATAFRAME
-      df.ardl = cbind(lagdata, dlag0)
+    #INITIAL ARDL MODEL DATAFRAME
+    df.ardl = cbind(lagdata, dlag0)
 
-      #ID FOR TREND
-      df.ardl$timeid = 1:nrow(df.ardl)
+    #ID FOR TREND
+    df.ardl$timeid = 1:nrow(df.ardl)
 
-      #FORMULA BASED ON CASE
-      text.ardl = (paste(colnames(df.ardl)[(1 + d)], "~", paste(colnames(df.ardl)[-(1 +
-                                                                                      d)], collapse = '+')))
+    #FORMULA BASED ON CASE
+    text.ardl = (paste(colnames(df.ardl)[(1 + d)], "~", paste(colnames(df.ardl)[-(1 +
+                                                                                    d)], collapse = '+')))
 
+    if (case == 1) {
+      formula.ardl = as.formula(paste(substr(text.ardl, 1, nchar(text.ardl) -
+                                               7), "-1"))
+    }
+
+    if (case == 2) {
+      formula.ardl = as.formula(paste(substr(text.ardl, 1, nchar(text.ardl) -
+                                               7)))
+    }
+
+    if (case == 3) {
+      formula.ardl = as.formula(paste(substr(text.ardl, 1, nchar(text.ardl) -
+                                               7)))
+    }
+
+    if (case >= 4) {
+      formula.ardl = as.formula(text.ardl)
+    }
+
+    if(is.null(fix.ardl)){
       if (case == 1) {
-        formula.ardl = as.formula(paste(substr(text.ardl, 1, nchar(text.ardl) -
-                                                 7), "-1"))
         fixed = c(-1, rep(0, d), rep(-1, (d - 1)))
       }
 
       if (case == 2) {
-        formula.ardl = as.formula(paste(substr(text.ardl, 1, nchar(text.ardl) -
-                                                 7)))
         fixed = c(-1, rep(0, d), rep(-1, (d - 1)))
       }
 
       if (case == 3) {
-        formula.ardl = as.formula(paste(substr(text.ardl, 1, nchar(text.ardl) - 7)))
         fixed = c(-1, rep(0, d), rep(-1, (d - 1)))
       }
 
       if (case >= 4) {
-        formula.ardl = as.formula(text.ardl)
         fixed = c(-1, rep(0, d), rep(-1, (d - 1)), 0)
       }
 
+      if(is.null(info.ardl) | !(info.ardl %in% c("AIC","AICc","BIC","R2","adjR2"))){
+        info.ardl="AIC"
+        warning("Default ARDL lag choice criterion: AIC")
+      }
+
       #SELECT BEST ARDL ORDER
       ardl_select = ARDL::auto_ardl(
         formula.ardl,
         data = na.omit(df.ardl),
         max_order = maxlag,
-        fixed_order = fixed
+        fixed_order = fixed,
+        selection=info.ardl
       )
       best_ardl = ardl_select$best_order[c(1, (d + 2):(2 * d))]
 
       #CONSTRUCT FINAL ARDL DATAFRAME
       final_dlag = lag_mts(dlag0, k = best_ardl)
+      formula.ardl2=ardl_select$best_model$full_formula
     } else{
-      if (length(difflags) != d) {
-        warning("number of variable is not equal to number of difflags, taking only the first value")
-        difflags0 = rep(difflags[1], d)
-        difflags = difflags0
+      if (length(fix.ardl) != d) {
+        warning("number of variable is not equal to number of fix.ardl, taking only the first value")
+        difflag0 = rep(fix.ardl[1], d)
+        fix.ardl = difflag0
       }
-      final_dlag = lag_mts(dlag0, k = difflags)
+
+      final_dlag = lag_mts(dlag0, k = fix.ardl)
+      best_ardl = fix.ardl
+      text.ardl2 = paste0(colnames(final_dlag),collapse = '+')
+      formula.ardl2=paste(as.character(formula.ardl)[2],
+                          as.character(formula.ardl)[1],
+                          as.character(formula.ardl)[3],"+",
+                          text.ardl2)
     }
 
     df.ardl.l = list(df.ardl.c = na.omit(cbind(dlag0[, 1], lagdata, final_dlag, dlag0[, -1])),
@@ -196,11 +249,72 @@ boot_ardl =
       pss.fi01 = pss.fi02 = pss.fi05 = pss.fi10 = list()
 
     #EMPTY OBJECTS FOR BOOT PROCEDURE
-    F.overallc = t.depc = F.indc = matrix(0, nrow = 2, ncol = B)
-    quantFOV = quantFIND = quantt = matrix(0, nrow = length(crit.H0), ncol = 2)
+    F.overallc = t.depc = F.indc = matrix(0, nrow = 2, ncol = nboot)
+    quantFOV = quantFIND = quantt = matrix(0, nrow = length(a.boot.H0), ncol = 2)
+
+
+    typevecm = ifelse(case < 2, "none", ifelse(case < 4, "const", "both"))
+
+    #SELECT UNRESTRICTED VECM
+    if(is.null(fix.vecm)){
+
+      if(is.null(info.vecm) | !(info.vecm %in% c("AIC","HQ","SC","FPE"))){
+        info.vecm="AIC"
+        idv = 1
+        warning("Default VECM lag choice criterion: AIC")
+      }else{
+        idv = 1 * (info.vecm == "AIC") +
+          2 * (info.vecm == "HQ") +
+          3 * (info.vecm == "SC") +
+          4 * (info.vecm == "FPE")
+      }
+
+      vecm = vars::VARselect(
+        na.omit(dlag0),
+        lag.max = maxlag,
+        type = typevecm,
+        exogen = na.omit(lagdata))
+
+      vecmsel = vecm$selection[idv]
+    }else{
+      vecmsel=fix.vecm
+    }
+
+    #CONFIRM UNRESTRICTED VECM
+    vecm = vars::VAR(
+      na.omit(dlag0),
+      p = vecmsel,
+      type = typevecm,
+      exogen = na.omit(lagdata)
+    )
+
+    summary(vecm)
+
+    nx = nrow(na.omit(dlag0))
+    dx = d * vecmsel + d * (d - 1)
+
+    #IMPOSE WEAK EXOGENEITY AND REMOVE
+    #NON-SIGNIFICANT SHORT-RUN ESTIMATES
+    vecm.ser = vars::restrict(vecm,
+                              method = "ser",
+                              thresh = qnorm(1 - a.vecm))
+    mat.ser = vecm.ser$restrictions
+    mat.ser[,(ncol(mat.ser) - d + 1)] = c(1, rep(0, d - 1))
+    if(typevecm=="const") mat.ser[, (ncol(mat.ser) - d)] = 1
+    if(typevecm=="trend") mat.ser[, (ncol(mat.ser) - d)] = 1
+    if(typevecm=="both"){
+      mat.ser[, (ncol(mat.ser) - d)] = 1
+      mat.ser[, (ncol(mat.ser) - d - 1)] = 1
+    }
+    mat.ser[, (ncol(mat.ser) - d + 2):ncol(mat.ser)] = 1
+    vecm.ser = vars::restrict(vecm, method = "manual", resmat = mat.ser)
+
+    vecm.out=vecm.ser
+    
+    H0=list(H0.c = c("fov","t","find"), H0.uc = "find")
 
     for (cond in 1:2) {
-      formula.ardl = ardl_select$best_model$full_formula
+      formula.ardlx = as.formula(formula.ardl2)
       conditional = ifelse(cond == 1, "Conditional", "Unconditional")
       colnames(df.ardl.l[[cond]])[1] = colnames(dlag0)[1]
 
@@ -209,14 +323,14 @@ boot_ardl =
         df.ardl.l[[cond]]$timeid = df.ardl$timeid
       }
 
-      formula.ardl = ardl_to_lm(formula.ardl, case = case, d = d)
+      formula.ardlx = ardl_to_lm(formula.ardlx, case = case, d = d)
 
       if (cond == 2) {
-        formula.ardl = strip_formula_UC(formula.ardl,case,d)
+        formula.ardlx = strip_formula_UC(formula.ardlx,case,d)
       }
 
       #ESTIMATE UNRESTRICTED ARDL MODEL
-      mod.ardl.l[[cond]] = lm(formula.ardl, data = na.omit(df.ardl.l[[cond]]))
+      mod.ardl.l[[cond]] = lm(formula.ardlx, data = na.omit(df.ardl.l[[cond]]))
       beta.ardl.l[[cond]] = coef(mod.ardl.l[[cond]])
 
       #OMEGA DEPENDS ON CONDITIONING
@@ -268,7 +382,7 @@ boot_ardl =
         object.out = T
       )
 
-      smk_crit = bootCT::smk_crit
+      #smk_crit = bootCT::smk_crit
       smk_test = NA
 
       nsmk = 81
@@ -308,11 +422,12 @@ boot_ardl =
         pss.fi10[[cond]] = c(smk_test[4, 3], smk_test[4, 4])
       }
 
-      for (sh in c("fov", "t", "find")) {
-        H0=sh
+      h=H0[[cond]]
+
+      for (hid in 1:length(h)) {
         #F OVERALL IMPOSING THE NULL
-        if (H0 == "fov") {
-          formula.ardl.H0 = strip_formula_H0(formula.ardl, H0, case, d)
+        if (h[hid] == "fov") {
+          formula.ardl.H0 = strip_formula_H0(formula.ardlx, h[hid], case, d)
           mod.ardl.H0 = lm(formula.ardl.H0, data = df.ardl.l[[cond]])
 
           beta.ardl.H0 = coef(mod.ardl.H0) #RESTRICTED ESTIMATES
@@ -328,40 +443,6 @@ boot_ardl =
           #RESTRICTED RESIDUALS
           e.ardl.H0 = residuals(mod.ardl.H0)
 
-          typevecm = ifelse(case == 1, "none", ifelse(case < 4, "const", "both"))
-
-          #SELECT UNRESTRICTED VECM
-          vecm = vars::VARselect(
-            na.omit(dlag0),
-            lag.max = maxlag,
-            type = typevecm,
-            exogen = na.omit(lagdata)
-          )
-
-          vecmsel = min(vecm$selection)
-
-          #CONFIRM UNRESTRICTED VECM
-          vecm = vars::VAR(
-            na.omit(dlag0),
-            p = vecmsel,
-            type = typevecm,
-            exogen = na.omit(lagdata)
-          )
-
-          nx = nrow(na.omit(dlag0))
-          dx = d * vecmsel + d * (d - 1)
-
-          #IMPOSE WEAK EXOGENEITY AND REMOVE
-          #NON-SIGNIFICANT SHORT-RUN ESTIMATES
-          vecm.ser = vars::restrict(vecm,
-                                    method = "ser",
-                                    thresh = qnorm(1 - p.vecm))
-          mat.ser = vecm.ser$restrictions
-          mat.ser[, ncol(mat.ser) - d + 1] = c(1, rep(0, d - 1))
-          mat.ser[, ncol(mat.ser) - d] = 1
-          mat.ser[, (ncol(mat.ser) - d + 2):ncol(mat.ser)] = 1
-          vecm.ser = vars::restrict(vecm, method = "manual", resmat = mat.ser)
-
           #VECM RESIDUALS
           e.vecm = residuals(vecm.ser)
           if (length(e.ardl.H0) > nrow(e.vecm)) {
@@ -393,7 +474,7 @@ boot_ardl =
           #RESTRICTED BETA DEPENDING ON CASE
           if (case == 1) {
             hbeta.ardl.H0 = beta.ardl.H0[1:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[1:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[1:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -404,7 +485,7 @@ boot_ardl =
             azero[1] = 0
             #INTERCEPT CONSTRAINED TO VECM
             hbeta.ardl.H0 = beta.ardl.H0[1:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[1:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[1:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -414,7 +495,7 @@ boot_ardl =
           if (case == 3) {
             azero[1] = beta.ardl.H0[1] #ARDL UNCONSTRAINED INTERCEPT
             hbeta.ardl.H0 = beta.ardl.H0[2:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[2:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[2:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -426,7 +507,7 @@ boot_ardl =
             azero[1] = beta.ardl.H0[1] #ARDL UNCONSTRAINED INTERCEPT
             auno[1] = 0
             hbeta.ardl.H0 = beta.ardl.H0[2:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[2:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[2:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -437,7 +518,7 @@ boot_ardl =
             azero[1] = beta.ardl.H0[1] #ARDL UNCONSTRAINED INTERCEPT
             auno[1] = beta.ardl.H0[length(beta.ardl.H0)] #ARDL UNCONSTRAINED TREND
             hbeta.ardl.H0 = beta.ardl.H0[2:(length(beta.ardl.H0) - 1)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[2:(length(beta.ardl.H0) -
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[2:(length(beta.ardl.H0) -
                                                              1)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
@@ -452,8 +533,8 @@ boot_ardl =
           A[-1, 1] = rep(0, d - 1)
         }
 
-        if (H0 == "t") {
-          formula.ardl.H0 = strip_formula_H0(formula.ardl, H0, case,d)
+        if (h[hid] == "t") {
+          formula.ardl.H0 = strip_formula_H0(formula.ardlx, h[hid], case,d)
 
           mod.ardl.H0 = lm(formula.ardl.H0, data = df.ardl.l[[cond]]) #case III
           summary(mod.ardl.H0)
@@ -469,39 +550,8 @@ boot_ardl =
 
           e.ardl.H0 = residuals(mod.ardl.H0)
 
-          typevecm = ifelse(case == 1, "none", ifelse(case < 4, "const", "both"))
-
-          vecm = vars::VARselect(
-            na.omit(dlag0),
-            lag.max = maxlag,
-            type = typevecm,
-            exogen = na.omit(lagdata)
-          )
-
-          vecmsel = min(vecm$selection)
-
-          vecm = vars::VAR(
-            na.omit(dlag0),
-            p = vecmsel,
-            type = typevecm,
-            exogen = na.omit(lagdata)
-          )
-
-          nx = nrow(na.omit(dlag0))
-          dx = d * vecmsel + d * (d - 1)
-
-          vecm.ser = vars::restrict(vecm,
-                                    method = "ser",
-                                    thresh = qnorm(1 - p.vecm))
-          mat.ser = vecm.ser$restrictions
-          mat.ser[, ncol(mat.ser) - d + 1] = c(1, rep(0, d - 1))
-          mat.ser[, ncol(mat.ser) - d] = 1
-          mat.ser[, (ncol(mat.ser) - d + 2):ncol(mat.ser)] = 1
-          vecm.ser = vars::restrict(vecm, method = "manual", resmat = mat.ser)
-
-          summary(vecm.ser)
-
           e.vecm = residuals(vecm.ser)
+
           if (length(e.ardl.H0) > nrow(e.vecm)) {
             diffrow = length(e.ardl.H0) - nrow(e.vecm)
             e.z = cbind(e.ardl.H0[-c(1:diffrow)], e.vecm[,-1])
@@ -529,7 +579,7 @@ boot_ardl =
           if (case == 1) {
             ay.x = -beta.ardl.H0[1:(d - 1)]
             hbeta.ardl.H0 = beta.ardl.H0[d:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[d:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[d:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -539,7 +589,7 @@ boot_ardl =
             azero[1] = 0
             ay.x = -beta.ardl.H0[1:(d - 1)]
             hbeta.ardl.H0 = beta.ardl.H0[d:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[d:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[d:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -549,7 +599,7 @@ boot_ardl =
             azero[1] = beta.ardl.H0[1]
             ay.x = -beta.ardl.H0[2:d]
             hbeta.ardl.H0 = beta.ardl.H0[(d+1):length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[(d+1):length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[(d+1):length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -560,7 +610,7 @@ boot_ardl =
             auno[1] = 0
             ay.x = -beta.ardl.H0[2:d]
             hbeta.ardl.H0 = beta.ardl.H0[(d+1):length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[(d+1):length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[(d+1):length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -571,7 +621,7 @@ boot_ardl =
             auno[1] = beta.ardl.H0[length(beta.ardl.H0)]
             ay.x = -beta.ardl.H0[2:d]
             hbeta.ardl.H0 = beta.ardl.H0[(d+1):(length(beta.ardl.H0) - 1)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[(d+1):(length(beta.ardl.H0) - 1)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[(d+1):(length(beta.ardl.H0) - 1)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -584,9 +634,9 @@ boot_ardl =
           A[-1, 1] = rep(0, d - 1)
         }
 
-        if (H0 == "find") {
+        if (h[hid] == "find") {
 
-          formula.ardl.H0 = strip_formula_H0(formula.ardl, H0, case,d)
+          formula.ardl.H0 = strip_formula_H0(formula.ardlx, h[hid], case,d)
 
           mod.ardl.H0 = lm(formula.ardl.H0, data = df.ardl.l[[cond]]) #case III
           summary(mod.ardl.H0)
@@ -602,39 +652,6 @@ boot_ardl =
 
           e.ardl.H0 = residuals(mod.ardl.H0)
 
-          typevecm = ifelse(case == 1, "none", ifelse(case < 4, "const", "both"))
-
-          vecm = vars::VARselect(
-            na.omit(dlag0),
-            lag.max = maxlag,
-            type = typevecm,
-            exogen = na.omit(lagdata)
-          )
-
-          vecmsel = min(vecm$selection)
-
-          vecm = vars::VAR(
-            na.omit(dlag0),
-            p = vecmsel,
-            type = typevecm,
-            exogen = na.omit(lagdata)
-          )
-
-          summary(vecm)
-          nx = nrow(na.omit(dlag0))
-          dx = d * vecmsel + d * (d - 1)
-
-          vecm.ser = vars::restrict(vecm,
-                              method = "ser",
-                              thresh = qnorm(1 - p.vecm))
-          mat.ser = vecm.ser$restrictions
-          mat.ser[, ncol(mat.ser) - d + 1] = c(1, rep(0, d - 1))
-          mat.ser[, ncol(mat.ser) - d] = 1
-          mat.ser[, (ncol(mat.ser) - d + 2):ncol(mat.ser)] = 1
-          vecm.ser = vars::restrict(vecm, method = "manual", resmat = mat.ser)
-
-          summary(vecm.ser)
-
           e.vecm = residuals(vecm.ser)
           if (length(e.ardl.H0) > nrow(e.vecm)) {
             diffrow = length(e.ardl.H0) - nrow(e.vecm)
@@ -663,7 +680,7 @@ boot_ardl =
           if (case == 1) {
             ayy = -beta.ardl.H0[1]
             hbeta.ardl.H0 = beta.ardl.H0[2:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[2:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[2:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -673,7 +690,7 @@ boot_ardl =
             azero[1] = 0
             ayy = -beta.ardl.H0[1]
             hbeta.ardl.H0 = beta.ardl.H0[2:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[2:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[2:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -683,7 +700,7 @@ boot_ardl =
             azero[1] = beta.ardl.H0[1]
             ayy = -beta.ardl.H0[2]
             hbeta.ardl.H0 = beta.ardl.H0[3:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[3:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[3:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -694,7 +711,7 @@ boot_ardl =
             auno[1] = 0
             ayy = -beta.ardl.H0[2]
             hbeta.ardl.H0 = beta.ardl.H0[3:length(beta.ardl.H0)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[3:length(beta.ardl.H0)]
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[3:length(beta.ardl.H0)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             GAMMAX[1,-join_nameG] = 0
@@ -705,7 +722,7 @@ boot_ardl =
             auno[1] = beta.ardl.H0[length(beta.ardl.H0)]
             ayy = -beta.ardl.H0[2]
             hbeta.ardl.H0 = beta.ardl.H0[3:(length(beta.ardl.H0) - 1)] *
-              (summary(mod.ardl.H0)[[4]][, 4] < p.ardl)[3:(length(beta.ardl.H0) -
+              (summary(mod.ardl.H0)[[4]][, 4] < a.ardl)[3:(length(beta.ardl.H0) -
                                                              1)]
             join_nameG = which(names(GAMMAX[1,]) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
             join_nameB = which(names(hbeta.ardl.H0) %in% intersect(names(GAMMAX[1,]), names(hbeta.ardl.H0)))
@@ -721,7 +738,7 @@ boot_ardl =
         }
 
         #BOOT PROCEDURE
-        for (b in 1:B) {
+        for (b in 1:nboot) {
           #RESIDUAL RESAMPLING
           resid.b = (dplyr::sample_n(
             as.data.frame(e.z),
@@ -780,7 +797,6 @@ boot_ardl =
                          as.character(
                             diff.df[1:(d * (vecmsel + 1)),]
                          ))
-          cnames = c(lev.col, d.col, "const")
           colnames(boot.data) = c("y", paste0("x", 1:(d - 1)))
 
           #LAGS AND DIFFERENCES
@@ -847,29 +863,29 @@ boot_ardl =
           Sigmabeta.ardl.b = summary(mod.ardl.b)$cov.unscaled * pracma::repmat((sigmae.ardl.b), length(coef(mod.ardl.b)))
 
           #BOOT STATISTICS
-          if (H0 == "fov") {
+          if (h[hid] == "fov") {
             wtestfo = aod::wald.test(Sigmabeta.ardl.b, beta.ardl.b, Terms = terms.ur)
             F.overallc[cond, b] = wtestfo$result$chi2[1] / wtestfo$result$chi2[2]
           }
-          if (H0 == "t") {
+          if (h[hid] == "t") {
             t.depc[cond, b] = (beta.ardl.b[(1 + 1 * (case > 1))] / sqrt(Sigmabeta.ardl.b[(1 +
                                                                                             1 * (case > 1)), (1 + 1 * (case > 1))]))
           }
-          if (H0 == "find") {
+          if (h[hid] == "find") {
             wtestfi = aod::wald.test(Sigmabeta.ardl.b, beta.ardl.b, Terms = ((2:d) + ((case > 1) * 1)))
             F.indc[cond, b] = wtestfi$result$chi2[1] / wtestfi$result$chi2[2]
           }
 
           #PRINT MESSAGE
-          if (print & B>=200) {
-            signal = floor(B / 200)
+          if (print & nboot>=200) {
+            signal = floor(nboot / 200)
             if (!(b %% signal)) {
-              prog = floor(b / B * 50)
+              prog = floor(b / nboot * 50)
               cat(
                 '\r',
                 conditional,
                 " ",
-                H0,
+                h[hid],
                 ": [",
                 strrep("#", prog),
                 strrep(" ", 50 - prog),
@@ -878,11 +894,10 @@ boot_ardl =
                 "%",
                 sep = ""
               )
-              flush.console()
+              utils::flush.console()
             }
-            ## Pretend that some work is being done.
             Sys.sleep(0.001)
-            if (b == B) {
+            if (b == nboot) {
               cat(" \n\r")
             }
           }
@@ -890,27 +905,75 @@ boot_ardl =
       }
 
       if (sum(F.overallc[cond, ]) != 0) {
-        quantFOV[, cond] = quantile(F.overallc[cond, ], probs = 1 - crit.H0)
+        quantFOV[, cond] = quantile(F.overallc[cond, ], probs = 1 - a.boot.H0)
       }
       if (sum(t.depc[, cond]) != 0) {
-        quantt[, cond] = quantile(t.depc[cond, ], probs = crit.H0)
+        quantt[, cond] = quantile(t.depc[cond, ], probs = a.boot.H0)
       }
       if (sum(F.indc[cond, ]) != 0) {
-        quantFIND[, cond] = quantile(F.indc[cond, ], probs = 1 - crit.H0)
+        quantFIND[, cond] = quantile(F.indc[cond, ], probs = 1 - a.boot.H0)
+      }
+      if(cond==1){
+        findflag.c =rep(unlist(F.ind.os)[1],length(a.boot.H0))>quantFIND[,1]
+        fakecoint = cbind(a.boot.H0,0+(findflag.c))
+
+        if(all(fakecoint[,2]==0)){
+          break
+        }else{
+          print("Computing Find test on the UC model to detect spurious cointegration")
+          computeuc = 1
+        }
       }
 
     }
+    
+    #ENRICH OUTPUT IF UNCONDITIONAL MODEL IS CONSIDERED
+    if(computeuc == 1){
+      findflag.uc = rep(unlist(F.ind.os)[2],length(a.boot.H0))<quantFIND[,2]
+      flagspur = (findflag.uc & findflag.c) + 0
+      find.stat = unlist(F.ind.os)
+      quant.find = data.frame(cbind(a.boot.H0, quantFIND[,1], quantFIND[,2]))
+    }else{
+      find.stat = F.ind.os[[1]]
+      quant.find = data.frame(cbind(a.boot.H0, quantFIND[,1]))
+    }
+
+    data.out=data.frame(numdata)
+
+    #extra: Johansen tests on X 
+    
+    if(case==1) ecdet="none"
+    if(case==2) ecdet="const"
+    if(case==3) ecdet="none"
+    if(case==4) ecdet="trend"
+    if(case==5) ecdet="none"
+
+    VECM.ctrace = urca::ca.jo(numdata[,-1],
+                            ecdet=ecdet,
+                            K=vecmsel+1,
+                            spec="transitory",
+                            type="trace")
+
+    VECM.ceigen = urca::ca.jo(numdata[,-1],
+                            ecdet=ecdet,
+                            K=vecmsel+1,
+                            spec="transitory",
+                            type="eigen")
 
     results = list(
-      ARDL = mod.ardl.l,
+      data = data.out,
+      ARDL = mod.ardl.l[[1]],
+      VECM = vecm.out,
+      jo.test = list(trace=VECM.ctrace,eigen=VECM.ceigen),
       pssbounds = pss.b,
       smgbounds = smk_test,
-      fov.st = unlist(F.overall.os),
-      t.st = unlist(t.dep.os),
-      find.st = unlist(F.ind.os),
-      quantfov = (cbind(crit.H0, quantFOV)),
-      quantt = data.frame(cbind(crit.H0, quantt)),
-      quantfind = data.frame(cbind(crit.H0, quantFIND))
+      fov.stat = F.overall.os[[1]],
+      t.stat = t.dep.os[[1]],
+      find.stat = find.stat,
+      quantfov = data.frame(cbind(a.boot.H0, quantFOV[,1])),
+      quantt = data.frame(cbind(a.boot.H0, quantt[,1])),
+      quantfind = quant.find,
+      fakecoint = flagspur
     )
 
     class(results) = "bootCT"
